@@ -16,6 +16,7 @@ import Text.Blaze (preEscapedString)
 import Text.Blaze.Internal (preEscapedString)
 #endif
 import Text.Printf
+import Text.JSON
 
 import Hledger.Utils
 import Hledger.Data
@@ -47,7 +48,7 @@ $maybe m' <- msg
 
 -- | The sidebar used on most views.
 sidebar :: ViewData -> HtmlUrl AppRoute
-sidebar vd@VD{..} = accountsReportAsHtml opts vd $ accountsReport (reportopts_ $ cliopts_ opts) am j
+sidebar vd@VD{..} = accountsReportAsHtml opts vd $ accountsReport (reportopts_ $ cliopts_ opts){empty_=True} am j
 
 -- -- | Navigation link, preserving parameters and possibly highlighted.
 -- navlink :: ViewData -> String -> AppRoute -> String -> HtmlUrl AppRoute
@@ -111,22 +112,30 @@ searchform VD{..} = [hamlet|
 
 -- | Add transaction form.
 addform :: Text -> ViewData -> HtmlUrl AppRoute
-addform staticRootUrl vd@VD{..} = [hamlet|
+addform _ vd@VD{..} = [hamlet|
 <script type=text/javascript>
  \$(document).ready(function() {
-    /* dhtmlxcombo setup */
-    window.dhx_globalImgPath="#{staticRootUrl}/";
-    var desccombo  = new dhtmlXCombo("description");
-    var acct1combo = new dhtmlXCombo("account1");
-    var acct2combo = new dhtmlXCombo("account2");
-    desccombo.enableFilteringMode(true);
-    acct1combo.enableFilteringMode(true);
-    acct2combo.enableFilteringMode(true);
-    desccombo.setSize(300);
-    acct1combo.setSize(300);
-    acct2combo.setSize(300);
-    /* desccombo.enableOptionAutoHeight(true, 20); */
-    /* desccombo.setOptionHeight(200); */
+    /* select2 setup */
+    var param = {
+      "width": "250px",
+      "openOnEnter": false,
+      // createSearchChoice allows to create new values not in the options
+      "createSearchChoice":function(term, data) {
+        if ( $(data).filter( function() {
+                return this.text.localeCompare(term)===0;
+                }).length===0) {
+          return {text:term};
+        }
+      },
+      // id is what is passed during post
+      "id": function(object) {
+        return object.text;
+      }
+    };
+    \$("#description").select2($.extend({}, param, {data: #{toSelectData descriptions} }));
+    var accountData = $.extend({}, param, {data: #{toSelectData acctnames} });
+    \$("#account1").select2(accountData);
+    \$("#account2").select2(accountData);
  });
 
 <form#addform method=POST style=display:none;>
@@ -143,10 +152,7 @@ addform staticRootUrl vd@VD{..} = [hamlet|
        <td style=padding-left:1em;>
         Description:
        <td>
-        <select id=description name=description>
-         <option>
-         $forall d <- descriptions
-          <option value=#{d}>#{d}
+        <input type=hidden id=description name=description>
       <tr.helprow>
        <td>
        <td>
@@ -171,16 +177,16 @@ addform staticRootUrl vd@VD{..} = [hamlet|
   deschelp = "eg: supermarket (optional)" :: String
   date = "today" :: String
   descriptions = sort $ nub $ map tdescription $ jtxns j
+  acctnames = sort $ journalAccountNamesUsed j
+  -- Construct data for select2. Text must be quoted in a json string.
+  toSelectData as  = preEscapedString $ encode $ JSArray $ map (\a -> JSObject $ toJSObject [("text", showJSON a)]) as
   manyfiles = (length $ files j) > 1
   postingfields :: ViewData -> Int -> HtmlUrl AppRoute
   postingfields _ n = [hamlet|
 <tr#postingrow>
  <td align=right>#{acctlabel}:
  <td>
-  <select id=#{acctvar} name=#{acctvar}>
-   <option>
-   $forall a <- acctnames
-    <option value=#{a} :shouldselect a:selected>#{a}
+  <input type=hidden id=#{acctvar} name=#{acctvar}>
  ^{amtfield}
 <tr.helprow>
  <td>
@@ -191,11 +197,9 @@ addform staticRootUrl vd@VD{..} = [hamlet|
   <span.help>#{amthelp}
 |]
    where
-    shouldselect a = n == 2 && maybe False ((a==).fst) (inAccount qopts)
     withnumber = (++ show n)
     acctvar = withnumber "account"
     amtvar = withnumber "amount"
-    acctnames = sort $ journalAccountNamesUsed j
     (acctlabel, accthelp, amtfield, amthelp)
        | n == 1     = ("To account"
                      ,"eg: expenses:food"
@@ -397,7 +401,7 @@ journalTransactionsReportAsHtml _ vd (_,items) = [hamlet|
    itemAsHtml VD{..} (n, _, _, _, (t, _, split, _, amt, _)) = [hamlet|
 <tr.item.#{evenodd}.#{firstposting}>
  <td.date>#{date}
- <td.description colspan=2 title="#{show t}">#{elideRight 60 desc}
+ <td.description colspan=2>#{elideRight 60 desc}
  <td.amount align=right>
   $if showamt
    #{mixedAmountAsHtml amt}
@@ -419,8 +423,8 @@ $forall p' <- tpostings t
 
 -- Generate html for an account register, including a balance chart and transaction list.
 registerReportHtml :: WebOpts -> ViewData -> TransactionsReport -> HtmlUrl AppRoute
-registerReportHtml opts vd r@(_,items) = [hamlet|
- ^{registerChartHtml items}
+registerReportHtml opts vd r = [hamlet|
+ ^{registerChartHtml $ map snd $ transactionsReportByCommodity r}
  ^{registerItemsHtml opts vd r}
 |]
 
@@ -482,8 +486,8 @@ $forall p' <- tpostings t
                --                      Data.Foldable.Foldable t1 =>
                --                      t1 (Transaction, t2, t3, t4, t5, MixedAmount)
                --                      -> t -> Text.Blaze.Internal.HtmlM ()
-registerChartHtml :: [TransactionsReportItem] -> HtmlUrl AppRoute
-registerChartHtml items =
+registerChartHtml :: [[TransactionsReportItem]] -> HtmlUrl AppRoute
+registerChartHtml itemss =
  -- have to make sure plot is not called when our container (maincontent)
  -- is hidden, eg with add form toggled
  [hamlet|
@@ -495,10 +499,13 @@ registerChartHtml items =
    if (chartdiv.is(':visible'))
      \$.plot(chartdiv,
              [
-              [
-               $forall i <- items
-                [#{dayToJsTimestamp $ triDate i}, #{triBalance i}],
-              ]
+              $forall items <- itemss
+               [
+                $forall i <- reverse items
+                 [#{dayToJsTimestamp $ triDate i}, #{triSimpleBalance i}],
+                []
+               ],
+              []
              ],
              {
                xaxis: {
@@ -528,7 +535,7 @@ numberTransactionsReportItems items = number 0 nulldate items
           (prevdy,prevdm,_) = toGregorian prevd
 
 mixedAmountAsHtml :: MixedAmount -> Html
-mixedAmountAsHtml b = preEscapedString $ addclass $ intercalate "<br>" $ lines $ showMixedAmount b
+mixedAmountAsHtml b = preEscapedString $ addclass $ intercalate "<br>" $ lines $ showMixedAmountWithoutPrice b
     where addclass = printf "<span class=\"%s\">%s</span>" (c :: String)
           c = case isNegativeMixedAmount b of Just True -> "negative amount"
                                               _         -> "positive amount"
