@@ -7,13 +7,17 @@ module Main where
 import Import
 import Yesod.Default.Config
 import Yesod.Test
+import Test.Hspec (hspec)
 import Application (makeFoundation)
 
 import HomeTest
 
 main :: IO ()
 main = do
-    conf <- loadConfig $ (configSettings Testing) { csParseExtra = parseExtra }
+    conf <- Yesod.Default.Config.loadConfig $ (configSettings Testing)
+                    { csParseExtra = parseExtra
+                    }
     foundation <- makeFoundation conf
-    app <- toWaiAppPlain foundation
-    runTests app (error "No database available") homeSpecs
+    hspec $ do
+      yesodSpec foundation $ do
+        homeSpecs
